#!/usr/bin/perl 
##########################################################################
#                                                                        #
# Perl Script for Resend lost Accouting packet to portaone Radius Server #
#                                                                        #
# Switzernet(c)2011                                                      #
#                                                                        #
##########################################################################
# my modules
use strict;
use warnings;
use Sys::Syslog;
use POSIX;
use Config::IniFiles;
use Switch;
use DBI;
use Date::Manip;
use Time::Local;
#use Asterisk::AMI;
use Asterisk::Manager;
##########################################################################
# Setting
my $lock_file = '/var/run/ast-send-notify.pid';
my $runas_user = 'root';
my $config_dir = '/etc/astrad/config';
##########################################################################
# Globals
my ($astman,$ast_hostname,$ast_username,$ast_password);
my ($dbh,$DB_NAME,$DB_HOST,$DB_USER,$DB_PASS);
my @peers;
##########################################################################
#                             Initialization                             #
##########################################################################
# Check if already running
if( -e $lock_file ) {
	open(PID,$lock_file);
	my $pid=<PID>;
	close PID;
	chomp $pid;
	if( !-e "/proc/$pid" ) {
		print STDERR "Lock file present, but no process with pid=$pid.\n";
		die "Can't delete lock file $lock_file\n" if !unlink $lock_file;
		print STDERR "Lock file has been removed.\n";
	} else {
		die "Lockfile present, another copy is punning pid=$pid\n";
	}
}
my ($name, $passwd, $uid, $gid) = getpwnam($runas_user) or die "$runas_user not in passwd file";;
##########################################################################
# set C locale
POSIX::setlocale(LC_ALL, "C");
##########################################################################
# Become daemon
my $pid;
if( !defined($pid = fork()) ) {
    die "cannot fork: $!";
} elsif ($pid) {
    # Create lockfile, and finish parent process
    open(PID, "> $lock_file") || die "ast-send-notify.pl: Unable to create lockfile $lock_file\n";
    print PID "$pid";
    close PID;
    chown $uid, $gid, $lock_file;
    exit;
} else {
    # daemon
    setpgrp();
    select(STDERR); $| = 1;
    select(STDOUT); $| = 1;
    openlog('ast-send-notify', 'cons,pid', 'daemon');
    syslog('notice', "Ast-Not: Resend lost Accounting packets for Asterisk started");
}

##########################################################################
# Install signall handler
$SIG{INT} = \&safe_exit;
$SIG{QUIT} = \&safe_exit;
$SIG{TERM} = \&safe_exit;
$SIG{HUP} = \&load_config;
##########################################################################
# Drop privileges
setuid($uid);
$< = $uid;
$> = $uid;
##########################################################################
# Signal Handlers
sub safe_exit {
    my($sig) = @_;
    syslog('crit', "Ast-Not: Caught a SIG - shutting down");
    $dbh->disconnect if $dbh;
    $astman->disconnect if $astman;
    unlink $lock_file or syslog('crit', "Ast-Not: Unable to create lockfile $lock_file\n");
    closelog();     
    exit;
}


##########################################################################
#                                FUNCTION                                #
##########################################################################
# sample '09:16:05 GMT Sat Dec 11 2004'
sub format_date_sql_2_gmt {
    my ($sql_date) = @_;
    my $date = &ParseDate($sql_date);
    my $gmt_date = &Date_ConvTZ($date,"","GMT");

    return &UnixDate($gmt_date, "%H:%M:%S GMT %a %b %e %Y");
}


##########################################################################
# Load config from mysql.conf
sub load_config {
    my $conf=Config::IniFiles->new(-file => "$config_dir/switzer.conf");
    return 0 if !defined $conf ;
    $DB_NAME  = $conf->val('ASTER_DB','DB_NAME');
    $DB_HOST  = $conf->val('ASTER_DB','DB_HOST');
    $DB_USER  = $conf->val('ASTER_DB','DB_USER');
    $DB_PASS  = $conf->val('ASTER_DB','DB_PASS');      
    $ast_hostname = $conf->val('Manager','AMI_PERMIT');
    $ast_username = $conf->val('Manager','AMI_USER');
    $ast_password = $conf->val('Manager','AMI_SECRET');
    return 1;
}


##########################################################################
# Astrad Radius DB connect 
sub DB_connect {
    my $cop=0;
    while( 1 ) {
        $cop++;
        $dbh = DBI->connect_cached("DBI:mysql:$DB_NAME;host=$DB_HOST",$DB_USER, $DB_PASS); 
        if( $dbh ) {
            syslog('info', 'Ast-Not: Connected to Astrad DB');
            return 1;
            } 
        else {
            syslog('info', "Ast-Not: Connection to Astrad DB failed -> SLEEP $cop");
            return 0 if ($cop>=3);
            }
        sleep $cop;
        }
}
##########################################################################
# Load register request from Astrad Reg. DB
sub load_peers {
    return 0 unless(DB_connect());
    my $result;
    my $request="SELECT username FROM location3 WHERE username NOT IN (SELECT username FROM multiple_ua GROUP BY domain,username HAVING domain=(SELECT value FROM sipconfig WHERE name='ipaddr') AND COUNT(username)>1);";
    syslog('info', "Ast-Not: request = $request");
    my $sth = $dbh->prepare($request);
    return 0 if (!$sth->execute());
    syslog('info', "Ast-Not: sth->rows = ".$sth->rows);
    push(@peers,$result->{username}) while ($result = $sth->fetchrow_hashref);
    $sth->finish;
    return 1; 
}
##########################################################################
# AMI connect
sub AMI_connect {
if (!$astman) {
   $astman = new Asterisk::Manager;
   $astman->user($ast_username);
   $astman->secret($ast_password);
   $astman->host($ast_hostname);
   return 0 unless ($astman->connect);
}
return 1;
}
##########################################################################
# Send notify
sub Send_Notify {
my $VR=1;
my ($cnt,$cus)=(0,'');
foreach my $p (@peers) {
   $cnt++;  
   $cus.=$p.' ' ;
   if ( $cnt>30 && $astman->connect ) {
      $VR=0 unless ($astman->sendcommand('Action'  => 'COMMAND',
                                         'Command' => 'sip notify nat '.$cus, 1 ));
      ($cnt,$cus)=(0,'');
      }
   }

if ( $cnt && $cus && $astman->connect ) {
   $VR=0 unless ($astman->sendcommand('Action'  => 'COMMAND',
                                      'Command' => 'sip notify nat '.$cus, 1 ));
   ($cnt,$cus)=(0,'');
   }
undef(@peers);
return $VR;
}
##########################################################################
#                                 main()                                 #
##########################################################################
load_config();
sleep 30 while ( load_peers() && AMI_connect() && Send_Notify() );
safe_exit();
##########################################################################
#                                  END                                   #
##########################################################################
